/**************************************************************************************************
   USB_Driver
   This module is used for communications with the user through HID Terminal.
**************************************************************************************************/
#include "USB_Driver.h"
#include "Buffers.h"
#include "LED.h"



// Variables
char readbuff[64];
char writebuff[64];

USB_TxStruct USB_Queue;        // Transmit buffer FIFO and input/output pointers






/**************************************************************************************************
   This is the USB interrupt handler */

void USBInterrupt() iv IVT_INT_OTG_FS ics ICS_AUTO {
     USB_Interrupt_Proc();
}

/**************************************************************************************************
   Initialization */
void USB_Init(void) {
     //USBIE_bit = 0;
     //IPC11bits.USBIP = 7;
   
     HID_Enable(&readbuff,&writebuff);
     USB_TxQueueInit();
     //NVIC_IntEnable(IVT_INT_OTG_FS);  // Enable USB device interrupt
}

/**************************************************************************************************
   This routine initializes the transmit buffer FIFO in and out pointers */
void USB_TxQueueInit(void) {
     
     USB_Queue.in = 0;
     USB_Queue.out = 0;
}

/**************************************************************************************************
   This routine makes an entry into the transmit FIFO and updates the in pointer to the next
   available location. */
void USB_PutQueue(char *msg) {
     
     USB_Queue.q[USB_Queue.in++] = msg;
     if (USB_Queue.in > (USB_Queue_Size-1))
         USB_Queue.in = 0;
}

/**************************************************************************************************
   This routine checks the transmit buffer FIFO and, if it is not empty, sends the buffer to USB.
   If the buffer was successfully sent, the out pointer is updated to the next available location.
   */

// Call this function in main loop
void USB_CheckOut(void) {

     if (USB_Queue.in != USB_Queue.out) {          // Look for a pointer in the FIFO

          // This is a blocking function
          if (HID_Write(USB_Queue.q[USB_Queue.out],64) != 0) {
              USB_Queue.out++;
              if (USB_Queue.out > (USB_Queue_Size-1))  // If past the end of array
                  USB_Queue.out = 0;                   // set pointer to start of array
          }
     }
}

/**************************************************************************************************
   This routine checks USB for a receive message then decodes the first character for a command. */

// Call this function in main loop
void USB_CheckIn(void) {
     char *BufNum;

     if (HID_Read() != 0) {
          switch (readbuff[0]) {
               
               case 'a':  // Return a canned message that indicates everything is working OK
                   USB_PutQueue("Command a received\r\n");
                   break;
               case 'b':  // Set the yellow LED
                   LED_Set_Yellow();
                   USB_PutQueue("OK, yellow set\r\n");
                   break;
               case 'c':  // Clear the yellow LED
                   LED_Clr_Yellow();
                   USB_PutQueue("OK, yellow cleared\r\n");
                   break;
               case 'd':  // Set the red LED
                   LED_Set_Red();
                   USB_PutQueue("OK, red set\r\n");
                   break;
               case 'e':  // Clear the red LED
                   LED_Clr_Red();
                   USB_PutQueue("OK, red cleared\r\n");
                   break;
               case 'f':
                   break;
               case 'g':
                   break;
               case 'h':
                   break;
               case 'i':
                   break;
               case 'j':
                   break;
               case 'k':
                   break;
               case 'l':
                   break;
               case 'm':
                   break;
               case 'n':
                   break;
               case 'o':
                   break;
               case 'p':
                   break;
               case 'q':
                   break;
               case 'r':
                   break;
               case 's':
                   break;
               case 't':
                   break;
               case 'u':
                   break;
               case 'v':
                   break;
               case 'w':
                   break;
               case 'x':
                   break;
               case 'y':
                   break;
               case 'z':
                   break;
               
               // Etc.
              
               default:
                   USB_PutQueue("Unknown command: ");
                   
                   BufNum = GetBuffer();         // Get a free buffer
                   (void)sprintf(BufNum,"%c\r\n",readbuff[0]);
                   USB_PutQueue(BufNum);
                   break;
          }
     }
}